package no.statnett.ecp.pf;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;

public class Counter {
    private String type;
    private AtomicLong totalMs = new AtomicLong(0);
    private AtomicLong sleepMs = new AtomicLong(0);
    private AtomicInteger messageCount = new AtomicInteger(0);

    private double throughputMsgPrSec = 0;
    private List<String> debugInfo = new ArrayList<>();

    public Counter(String type) {
        this.type = type;
    }

    public Counter(String type, long totalMs, int messageCount, long sleepMs) {
        this.type = type;
        this.totalMs = new AtomicLong(totalMs);
        this.messageCount = new AtomicInteger(messageCount);
        this.sleepMs = new AtomicLong(sleepMs);
    }

    // Used to summarize several independent counters
    public double add(Counter counter) {
        long totalMsP = counter.getTotalMs();
        long sleepMsP = counter.sleepMs.get();
        int messageCountP = counter.getMessageCount();

        this.totalMs.addAndGet(totalMsP);
        this.sleepMs.addAndGet(sleepMsP);
        this.messageCount.addAndGet(messageCountP);

        double throughput = totalMsP == 0 ? 0D : (double) messageCountP / (totalMsP / 1000D);
//        System.out.println("Calculate " + type + " throughput " + throughput + " based on " + messageCountP + " messages in " + totalMsP + "ms");
        this.throughputMsgPrSec += throughput;
        return throughput;
    }

    public void addDebugInfo(String s) {
        debugInfo.add(s);
    }

    public List<String> getDebugInfo() {
        return debugInfo;
    }

    public int getMessageCount() {
        return messageCount.get();
    }

    public long getTotalMs() {
        return totalMs.get();
    }

    public long getSleepMs() {
        return sleepMs.get();
    }

    public void increase(long totalMs, int messageCount, long sleepMs) {
        this.totalMs.addAndGet(totalMs);
        this.messageCount.addAndGet(messageCount);
        this.sleepMs.addAndGet(sleepMs);
    }

    public int getThroughputKBS(int messageSizeKB) {
        return (int) (throughputMsgPrSec * messageSizeKB);
    }

    public Counter reset() {
        Counter copy = new Counter(type, totalMs.get(), messageCount.get(), sleepMs.get());
        totalMs.set(0);
        messageCount.set(0);
        sleepMs.set(0);
        return copy;
    }
}
