package no.statnett.ecp.pf;

import jakarta.jms.*;
import no.statnett.ecp.utils.Broker;

import java.util.Map;

public class MassConsumer implements ArtemisClient {

    private final Session session;
    private final MessageConsumer consumer;
    private int msgSizeB;
    private Counter counter = new Counter("Cons");

    public MassConsumer(Map<String, String> amqpConfig, String queueName, int msgSizeB) throws JMSException {
        Connection connection = Broker.createAndStartConnection(amqpConfig);
        session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
        consumer = session.createConsumer(session.createQueue(queueName), null);
        this.msgSizeB = msgSizeB;
    }


    public void setRate(int rate) {
        // consumer always consumes at max
    }

    public int getRate() {
        // consumer always consumes at max
        return 100;
    }

    public void decreaseRate() {
        // consumer always consumes at max
    }

    public void increaseRate() {
        // consumer always consumes at max
    }

    @Override
    public void setQueueSize(int queueSize) {
        // consumer does not care about the queue-size, it always consumes as much as it can
    }

    public Counter getCounter() {
        return counter;
    }

    @Override
    public void run() {
        try {
            long cycleStartTms = System.currentTimeMillis();
            int messageCount = 0;
            while (true) {
                // Mass-produce at a certain rate
                // The rate is variable, depending on the input from the Performance-class (Controller)
                // When 100 ms has passed, and the rate is between 0 and 100 (but not 0 or 100), we
                // calculate the time since last time we started sleeping, called cycleTime. We find
                // the wanted executionTime = rate (as percent) * cycleTime. sleepTime = cycleTime - executionTime
                long cycleTime = System.currentTimeMillis() - cycleStartTms;
                if (cycleTime > 100) {
                    cycleStartTms = System.currentTimeMillis(); // reset cycleTime immediately after dectecting end of previous cycle
                    counter.increase(cycleTime, messageCount, 0L);
                    messageCount = 0;
                }
                BytesMessage message = (BytesMessage) consumer.receive(100);
                if (message != null) {
                    messageCount++;
//          if (message.getBodyLength() != msgSizeB) {
//            System.err.println("Message size was " + message.getBodyLength() + "B, which is different from expected " + msgSizeB);
//          } else {
//            System.out.println("Message size of " + message.getBodyLength() + "B was consumed!!!");
//          }
                }
            }
        } catch (Throwable t) {
            System.out.println("Something happened - killed the mass-consumer: " + t);
        }
    }
}
